@@ -11,10 +11,12 @@ from alembic import autogenerate
 from alembic.migration import MigrationContext
 from alembic.testing import TestBase
 from alembic.testing import config
+from alembic.testing import assert_raises_message
 from alembic.testing.mock import Mock
 from alembic.testing.env import staging_env, clear_staging_env
 from alembic.testing import eq_
 from alembic.ddl.base import _fk_spec
+from alembic.util import CommandError
 
 py3k = sys.version_info >= (3, )
 
@@ -771,6 +773,32 @@ nullable=True))
                                            )
         assert not diff
 
+    def test_custom_type_compare(self):
+        class MyType(TypeDecorator):
+            impl = Integer
+
+            def compare_against_backend(self, dialect, conn_type):
+                return isinstance(conn_type, Integer)
+
+        diff = []
+        autogenerate.compare._compare_type(None, "sometable", "somecol",
+                                           Column("somecol", INTEGER()),
+                                           Column("somecol", MyType()),
+                                           diff, self.autogen_context
+                                           )
+        assert not diff
+
+        diff = []
+        autogenerate.compare._compare_type(None, "sometable", "somecol",
+                                           Column("somecol", String()),
+                                           Column("somecol", MyType()),
+                                           diff, self.autogen_context
+                                           )
+        eq_(
+            diff[0][0:4],
+            ('modify_type', None, 'sometable', 'somecol')
+        )
+
     def test_affinity_typedec(self):
         class MyType(TypeDecorator):
             impl = CHAR
@@ -1350,6 +1378,20 @@ class CompareMetadataTest(ModelOne, AutogenTest, TestBase):
         eq_(diffs[2][1][5], False)
         eq_(diffs[2][1][6], True)
 
+    def test_compare_metadata_as_sql(self):
+        context = MigrationContext.configure(
+            connection=self.bind.connect(),
+            opts={'as_sql': True}
+        )
+        metadata = self.m2
+
+        assert_raises_message(
+            CommandError,
+            "autogenerate can't use as_sql=True as it prevents "
+            "querying the database for schema information",
+            autogenerate.compare_metadata, context, metadata
+        )
+
 
 class PGCompareMetaData(ModelOne, AutogenTest, TestBase):
     __only_on__ = 'postgresql'
