@@ -1,20 +1,23 @@
 """Test op functions against MSSQL."""
 
-from unittest import TestCase
+from alembic.testing.fixtures import TestBase
 
 from sqlalchemy import Integer, Column
 
 from alembic import op, command, util
-from . import op_fixture, capture_context_buffer, \
-    _no_sql_testing_config, assert_raises_message, staging_env, \
-    three_rev_fixture, clear_staging_env, eq_
+
+from alembic.testing import eq_, assert_raises_message
+from alembic.testing.fixtures import capture_context_buffer, op_fixture
+from alembic.testing.env import staging_env, _no_sql_testing_config, \
+    three_rev_fixture, clear_staging_env
+from alembic.testing import config
 
 
-class FullEnvironmentTests(TestCase):
+class FullEnvironmentTests(TestBase):
 
     @classmethod
     def setup_class(cls):
-        env = staging_env()
+        staging_env()
         cls.cfg = cfg = _no_sql_testing_config("mssql")
 
         cls.a, cls.b, cls.c = \
@@ -46,7 +49,7 @@ class FullEnvironmentTests(TestCase):
         assert "BYE" in buf.getvalue()
 
 
-class OpTest(TestCase):
+class OpTest(TestBase):
 
     def test_add_column(self):
         context = op_fixture('mssql')
@@ -88,10 +91,10 @@ class OpTest(TestCase):
                         nullable=False)
         context.assert_('ALTER TABLE tests ALTER COLUMN col BIT NOT NULL')
 
+    @config.requirements.fail_before_sqla_084
     def test_drop_index(self):
         context = op_fixture('mssql')
         op.drop_index('my_idx', 'my_table')
-        # TODO: annoying that SQLA escapes unconditionally
         context.assert_contains("DROP INDEX my_idx ON my_table")
 
     def test_drop_column_w_default(self):
@@ -113,6 +116,11 @@ class OpTest(TestCase):
         op.alter_column("t", "c", server_default=False)
         context.assert_()
 
+    def test_drop_column_w_schema(self):
+        context = op_fixture('mssql')
+        op.drop_column('t1', 'c1', schema='xyz')
+        context.assert_contains("ALTER TABLE xyz.t1 DROP COLUMN c1")
+
     def test_drop_column_w_check(self):
         context = op_fixture('mssql')
         op.drop_column('t1', 'c1', mssql_drop_check=True)
